<?php

/**
 * @file
 * Administrative functions for the Translate set module.
 */

/**
 * Page callback: add or remove a string from the translate set.
 *
 * @param string $action
 *   The action to perform
 * @param int $lid
 *   The locale id.
 */
function translate_set_add_or_remove($action = '', $lid = 0) {
  $message = t('Invalid action');

  if (!empty($action) && !empty($lid)) {

    // Make sure this source exists.
    $exists = db_select('locales_source', 'ls')
      ->fields('ls', array('lid'))
      ->condition('lid', $lid)
      ->execute()
      ->fetchField();

    if ($exists) {
      switch ($action) {
        case 'remove':
          $message = t('The string has been removed from the translate set.');
          db_delete('locales_translate_set')
            ->condition('lid', $lid)
            ->execute();
          break;
        case 'add':
          $message = t('The string has been added to the translate set.');
          db_insert('locales_translate_set')
            ->fields(array('lid'), array($lid))
            ->execute();
          break;
      }
    }
  }

  drupal_set_message($message);
  drupal_goto('admin/config/regional/translate/translate');
}

/**
 * Page callback: String search screen.
 *
 * This function resembles locale_translate_seek_screen(), but we've replaced
 * _locale_translate_seek() by _translate_set_translate_seek() so we can
 * inject our own operation to add or remove a string to the translate set.
 */
function translate_set_translate_seek_screen() {
  // Add CSS.
  drupal_add_css(drupal_get_path('module', 'locale') . '/locale.css');

  // Load locale.admin.inc file for the filter form.
  module_load_include('inc', 'locale', 'locale.admin');

  $elements = drupal_get_form('locale_translation_filter_form');
  $output = drupal_render($elements);

  // Call our own translate seek function.
  $output .= translate_set_translate_seek();
  return $output;
}

/**
 * Perform a string search and display results in a table.
 *
 * @see translate_set_translate_seek_screen().
 */
function translate_set_translate_seek() {
  $output = '';

  // We have at least one criterion to match
  if (!($query = _locale_translate_seek_query())) {
    $query = array(
      'translation' => 'all',
      'group' => 'all',
      'language' => 'all',
      'string' => '',
    );
  }

  $sql_query = db_select('locales_source', 's');

  $limit_language = NULL;
  if ($query['language'] != 'en' && $query['language'] != 'all') {
    $sql_query->leftJoin('locales_target', 't', "t.lid = s.lid AND t.language = :langcode", array(':langcode' => $query['language']));
    $limit_language = $query['language'];
  }
  else {
    $sql_query->leftJoin('locales_target', 't', 't.lid = s.lid');
  }

  $sql_query->fields('s', array('source', 'location', 'context', 'lid', 'textgroup'));
  $sql_query->fields('t', array('translation', 'language'));

  // Compute LIKE section.
  switch ($query['translation']) {
    case 'translated':
      $sql_query->condition('t.translation', '%' . db_like($query['string']) . '%', 'LIKE');
      $sql_query->orderBy('t.translation', 'DESC');
      break;
    case 'untranslated':
      $sql_query->condition(db_and()
          ->condition('s.source', '%' . db_like($query['string']) . '%', 'LIKE')
          ->isNull('t.translation')
      );
      $sql_query->orderBy('s.source');
      break;
    case 'all' :
    default:
      $condition = db_or()
        ->condition('s.source', '%' . db_like($query['string']) . '%', 'LIKE');
      if ($query['language'] != 'en') {
        // Only search in translations if the language is not forced to English.
        $condition->condition('t.translation', '%' . db_like($query['string']) . '%', 'LIKE');
      }
      $sql_query->condition($condition);
      break;
  }

  // Add a condition on the text group.
  if (!empty($query['group']) && $query['group'] != 'all') {
    $sql_query->condition('s.textgroup', $query['group']);
  }

  $sql_query = $sql_query->extend('PagerDefault')->limit(50);
  $locales = $sql_query->execute();

  $groups = module_invoke_all('locale', 'groups');
  $header = array(t('Text group'), t('String'), t('Context'), ($limit_language) ? t('Language') : t('Languages'), array('data' => t('Operations'), 'colspan' => '3'));

  $strings = array();
  foreach ($locales as $locale) {
    if (!isset($strings[$locale->lid])) {
      $strings[$locale->lid] = array(
        'group' => $locale->textgroup,
        'languages' => array(),
        'location' => $locale->location,
        'source' => $locale->source,
        'context' => $locale->context,
      );
    }
    if (isset($locale->language)) {
      $strings[$locale->lid]['languages'][$locale->language] = $locale->translation;
    }
  }

  $rows = array();
  foreach ($strings as $lid => $string) {

    // Determine action for the Translate set link.
    $exists = db_select('locales_translate_set', 'ls')
      ->fields('ls', array('lid'))
      ->condition('lid', $lid)
      ->execute()
      ->fetchField();

    $translate_set_action = $exists ? 'remove' : 'add';
    $translate_set_action_label = $exists ? t('remove from translate set') : t('add to translate set');

    if (!isset($string['group'])) {
      $string['group'] = 'default';
    }

    $rows[] = array(
      $groups[$string['group']],
      array('data' => check_plain(truncate_utf8($string['source'], 150, FALSE, TRUE)) . '<br /><small>' . $string['location'] . '</small>'),
      $string['context'],
      array('data' => _locale_translate_language_list($string, $limit_language), 'align' => 'center'),
      array('data' => l(t('edit'), "admin/config/regional/translate/edit/$lid", array('query' => drupal_get_destination())), 'class' => array('nowrap')),
      array('data' => l(t('delete'), "admin/config/regional/translate/delete/$lid", array('query' => drupal_get_destination())), 'class' => array('nowrap')),
      array('data' => l($translate_set_action_label, "admin/config/regional/translate/translate-set-action/$translate_set_action/$lid"), 'class' => array('nowrap')),
    );
  }

  $output .= theme('table', array('header' => $header, 'rows' => $rows, 'empty' => t('No strings available.')));
  $output .= theme('pager');

  return $output;
}

/**
 * Page callback: Translate set screen.
 */
function translate_set_translate_form($form, $form_state) {
  $form = array();
  $rows = array();
  $languages = array();
  $number_of_strings = 0;
  $untranslated = array();
  $header = array(t('Source'));

  // Track new strings - do not run on a POST request.
  if (empty($_POST) && variable_get('translate_set_tracking_enabled', FALSE)) {
    translate_set_tracking_find_strings();
  }

  // Show all, untranslated or translated strings.
  $type = isset($_GET['type']) ? $_GET['type'] : 'all';

  // Filter on textgroup.
  $textgroup = isset($_GET['textgroup']) ? $_GET['textgroup'] : 'all';

  // Filter on context.
  $context = isset($_GET['context']) ? $_GET['context'] : '';

  // Search.
  $search = isset($_GET['search']) ? $_GET['search'] : '';

  // Check user language settings.
  $restricted_languages = db_select('locales_translate_set_user_languages', 'ltsul')
    ->fields('ltsul', array('language'))
    ->condition('uid', $GLOBALS['user']->uid)
    ->execute()
    ->fetchCol();

  // Determine the language.
  $language_list = language_list();
  $language_options = array();
  foreach ($language_list as $key => $info) {

    // Ignore built-in English.
    if ($key == 'en') {
      continue;
    }

    // Check restricted languages.
    if (!empty($restricted_languages)) {
      if (!in_array($info->language, $restricted_languages)) {
        continue;
      }
    }

    // Add to options.
    $language_options[$key] = $info->name . ' (' . $info->language . ')';
  }

  // If there are no options for the language, stop here.
  if (empty($language_options)) {
    $form['info'] = array(
      '#type' => 'markup',
      '#markup' => '<p>' . t('No languages found to translate to. Either add new languages or contact an administrator.') . '</p>',
    );
    return $form;
  }

  // Check selected language.
  $selected_language = isset($_GET['language']) ? (isset($language_options[$_GET['language']]) ? $_GET['language'] : $GLOBALS['language']->language) : $GLOBALS['language']->language;

  // Selected language can not be 'en' or a language you can not select.
  if ($selected_language == 'en' || !isset($language_options[$selected_language])) {
    $selected_language = key($language_options);
  }

  // Add to header and languages.
  // Languages is already an array since in the future we might allow
  // multiple columns as a feature.
  $header[] = $language_list[$selected_language]->name;
  $languages[] = $selected_language;

  // Create the query. We limit to around 500 strings so we don't hit the limit
  // of maximum post values in PHP. And also, 500 strings on a page is fine
  // enough.
  $select = db_select('locales_translate_set', 'lts')
    ->extend('PagerDefault')
    ->limit(round(500 / (count($header) -1)));

  $select->innerJoin('locales_source', 'ls', 'ls.lid = lts.lid');
  $select->fields('ls', array('lid', 'source', 'textgroup', 'context'));

  if ($textgroup != 'all') {
    $select->condition('ls.textgroup', $textgroup);
  }

  if (!empty($context)) {
    $select->condition('ls.context', $context);
  }

  if (!empty($search)) {
    $select->condition('ls.source', '%' . db_like($search) . '%', 'LIKE');
  }

  $select->orderBy('source', 'ASC');
  $strings = $select->execute();

  // Get the strings.
  foreach ($strings as $string) {
    $group = '';
    if ($string->textgroup != 'default') {
      $group = $string->textgroup . ' - ' . $string->context;
    }
    if (!isset($rows['string-' . $string->lid])) {
      $number_of_strings++;
      $rows['string-' . $string->lid] = array(
        '#type' => 'tree',
      );

      // Check the source, this can be a translated string from the
      // the fallback language or the source if non exists.
      $source = $string->source;
      $rows['string-' . $string->lid]['source'] = array(
        '#type' => 'markup',
        '#markup' => check_plain($source),
      );
    }

    foreach ($languages as $language) {
      $translation = db_select('locales_target', 'ls')
        ->fields('ls', array('translation'))
        ->condition('lid', $string->lid)
        ->condition('language', $language)
        ->execute()
        ->fetchField();

      $rows['string-' . $string->lid][$language] = array(
        '#type' => 'textarea',
        '#title' => $group,
        '#cols' => 250,
        '#rows' => 2,
        '#default_value' => $translation,
        '#attributes' => array('class' => array('translation-input')),
      );
      if (empty($translation)) {

        if (!isset($untranslated[$language])) {
          $untranslated[$language] = 0;
        }
        $untranslated[$language]++;

        // Depending on $type, remove this row or not.
        if ($type == 'translated') {
          $number_of_strings--;
          $untranslated[$language]--;
          unset($rows['string-' . $string->lid]);
        }

      }
      else {
        // Depending on $type, remove this row or not.
        if ($type == 'untranslated') {
          $number_of_strings--;
          unset($rows['string-' . $string->lid]);
        }
      }
    }
  }

  // Add stats.
  $untranslated_array = array();
  foreach ($untranslated as $count) {
    $untranslated_array[] = $count;
  }
  $untranslated_string = !empty($untranslated_array) ? implode(', ', $untranslated_array) : 0;
  $stats = format_string('Number of strings: @strings_number - Untranslated strings: @untranslated', array('@strings_number' => $number_of_strings, '@untranslated' => $untranslated_string));

  $form['stats'] = array(
    '#type' => 'markup',
    '#markup' => '<p>' . $stats . '</p>'
  );

  $form['container'] = array(
    '#type' => 'container',
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $form['container']['select_language'] = array(
    '#type' => 'select',
    '#title' => t('Language'),
    '#options' => $language_options,
    '#default_value' => $selected_language,
  );
  $form['container']['type'] = array(
    '#type' => 'select',
    '#title' => t('Filter'),
    '#options' => array('all' => t('All strings'), 'untranslated' => t('Untranslated strings'), 'translated' => t('Translated strings')),
    '#default_value' => $type,
  );
  $groups = module_invoke_all('locale', 'groups');
  $form['container']['textgroup'] = array(
    '#type' => 'select',
    '#title' => t('Textgroup'),
    '#options' => array_merge(array('all' => t('All text groups')), $groups),
    '#default_value' => $textgroup,
  );

  $contexts = array();
  $contexts_query = db_query('SELECT DISTINCT context FROM {locales_source} ORDER by context ASC');
  foreach ($contexts_query as $record) {
    $human_name = $record->context;
    if (empty($human_name)) {
      $human_name = t('All contexts');
    }
    $contexts[$record->context] = $human_name;
  }

  $form['container']['context'] = array(
    '#type' => 'select',
    '#title' => t('Context'),
    '#options' => $contexts,
    '#default_value' => $context,
  );
  $form['container']['search'] = array(
    '#type' => 'textfield',
    '#title' => 'search',
    '#default_value' => $search,
    '#size' => 20,
  );
  $form['container']['go'] = array(
    '#type' => 'submit',
    '#value' => t('Apply'),
    '#submit' => array('translate_set_filter_form_submit'),
  );
  $form['container']['reset'] = array(
    '#type' => 'submit',
    '#value' => t('Reset'),
    '#submit' => array('translate_set_filter_form_reset'),
  );

  if (!empty($rows)) {
    $rows['#tree'] = TRUE;
  }

  $form['table'] = array(
    '#theme' => 'translate_set_table',
    '#header' => $header,
    '#add_big_column_class' => TRUE,
    'strings' => $rows,
  );

  $form['pager'] = array(
    '#theme' => 'pager',
    '#weight' => 5,
  );

  if (!empty($rows)) {

    $form['#attached']['css'][] = drupal_get_path('module', 'translate_set') . '/translate_set.admin.css';

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => 'Save translations',
    );
  }

  return $form;
}

/**
 * Submit callback: Saves the translate set form.
 */
function translate_set_translate_form_submit($form, $form_state) {
  $stats = array(
    'updated' => 0,
    'created' => 0,
    'deleted' => 0,
  );
  $strings = $form_state['values']['strings'];
  $invalidate_languages = array();
  foreach ($strings as $string_lid => $translations) {
    list (, $lid) = explode('-', $string_lid);
    foreach ($translations as $language => $value) {
      $invalidate_languages[$language] = $language;
      $translation = db_select('locales_target', 'ls')
        ->fields('ls', array('translation'))
        ->condition('lid', $lid)
        ->condition('language', $language)
        ->execute()
        ->fetchField();

      if (!empty($value)) {
        // Only update or insert if we have a value to use.
        if (!empty($translation)) {
          $stats['updated']++;
          db_update('locales_target')
            ->fields(array(
              'translation' => $value,
            ))
            ->condition('lid', $lid)
            ->condition('language', $language)
            ->execute();
        }
        else {
          $stats['created']++;
          db_insert('locales_target')
            ->fields(array(
              'lid' => $lid,
              'translation' => $value,
              'language' => $language,
            ))
            ->execute();
        }
      }
      elseif (!empty($translation)) {
        $stats['deleted']++;
        // Empty translation entered: remove existing entry from database.
        db_delete('locales_target')
          ->condition('lid', $lid)
          ->condition('language', $language)
          ->execute();
      }
    }
  }

  // Invalidate caches and javascript translations.
  cache_clear_all('locale:', 'cache', TRUE);
  foreach ($invalidate_languages as $invalidate_language) {
    _locale_invalidate_js($invalidate_language);
  }

  drupal_set_message(t('Strings saved: @created created - @updated updated - @deleted deleted', array('@created' => $stats['created'], '@updated' => $stats['updated'], '@deleted' => $stats['deleted'])));
}

/**
 * Submit callback: filter the translate set form.
 */
function translate_set_filter_form_submit($form, &$form_state) {
  $form_state['redirect'] = array('admin/config/regional/translate/translate-set', array(
    'query' => array(
      'language' => $form_state['values']['select_language'],
      'type' => $form_state['values']['type'],
      'textgroup' => $form_state['values']['textgroup'],
      'search' => $form_state['values']['search'],
      'context' => $form_state['values']['context'],
    )
  ));
}

/**
 * Submit callback: reset the manage translate set form.
 */
function translate_set_filter_form_reset($form, &$form_state) {
  $form_state['redirect'] = 'admin/config/regional/translate/translate-set';
}

/**
 * Menu callback: present the manage the translate set form.
 */
function translate_set_manage_form($form, $form_state) {
  $rows = array();
  $header = array('', t('String'));
  $textgroup = isset($_GET['textgroup']) ? $_GET['textgroup'] : 'all';
  $only_in_set = isset($_GET['only_in_set']) ? $_GET['only_in_set'] : FALSE;

  $query = db_select('locales_source', 's')->extend('PagerDefault');
  $query->leftJoin('locales_translate_set', 'lts', "s.lid = lts.lid");
  $query->fields('s', array('source', 'lid'))
    ->fields('lts', array('lid'))
    ->orderBy('s.source');

  if ($textgroup != 'all') {
    $query->condition('s.textgroup', $textgroup);
  }

  if ($only_in_set) {
    $query->isNotNull('lts.lid');
  }

  $result = $query
    ->limit(50)
    ->execute();

  foreach ($result as $data) {

    $rows['string-' . $data->lid]['toggle'] = array(
      '#type' => 'checkbox',
      '#default_value' => $data->lts_lid ? TRUE : FALSE,
    );

    $rows['string-' . $data->lid]['source'] = array(
      '#type' => 'markup',
      '#markup' => check_plain($data->source),
    );

  }

  if (!empty($rows)) {
    $rows['#tree'] = TRUE;
  }

  $form['container'] = array(
    '#type' => 'container',
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
    '#weight' => -2
  );
  $groups = module_invoke_all('locale', 'groups');
  $form['container']['textgroup'] = array(
    '#type' => 'select',
    '#title' => t('Textgroup'),
    '#options' => array_merge(array('all' => t('All text groups')), $groups),
    '#default_value' => $textgroup,
  );
  $form['container']['only_in_set'] = array(
    '#type' => 'checkbox',
    '#title' => t('Only show strings in the set'),
    '#default_value' => $only_in_set,
  );
  $form['container']['go'] = array(
    '#type' => 'submit',
    '#value' => t('Apply'),
    '#submit' => array('translate_set_manage_filter_form_submit'),
  );
  $form['container']['reset'] = array(
    '#type' => 'submit',
    '#value' => t('Reset'),
    '#submit' => array('translate_set_manage_filter_form_reset'),
  );

  $form['table'] = array(
    '#theme' => 'translate_set_table',
    '#header' => $header,
    '#add_big_column_class' => FALSE,
    '#weight' => 0,
    'strings' => $rows,
  );

  if (!empty($rows)) {

    $form['#attached']['css'][] = drupal_get_path('module', 'translate_set') . '/translate_set.admin.css';

    $form['info'] = array(
      '#weight' => -3,
      '#type' => 'markup',
      '#markup' => '<p>' . t('Add or remove strings from the translate set by toggling the checkboxes.') . '</p>'
    );

    $form['pager_top'] = array(
      '#theme' => 'pager',
      '#weight' => -1,
    );

    $form['pager_bottom'] = array(
      '#theme' => 'pager',
      '#weight' => 1,
    );

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
      '#weight' => 2,
    );
  }

  return $form;
}

/**
 * Submit callback: saves the manage translate set form.
 */
function translate_set_manage_form_submit($form, $form_state) {
  $stats = array(
    'added' => 0,
    'deleted' => 0,
  );

  foreach ($form_state['values']['strings'] as $key => $data) {
    list(, $lid) = explode('-', $key);
    $toggled = $data['toggle'] === 0 ? FALSE : TRUE;
    $exists = db_select('locales_translate_set', 'ls')
      ->fields('ls', array('lid'))
      ->condition('lid', $lid)
      ->execute()
      ->fetchField();

    // Deleted.
    if ($exists && !$toggled) {
      $stats['deleted']++;
      db_delete('locales_translate_set')
        ->condition('lid', $lid)
        ->execute();
    }
    elseif (!$exists && $toggled) {
      $stats['added']++;
      db_insert('locales_translate_set')
        ->fields(array('lid'), array($lid))
        ->execute();
    }
  }

  drupal_set_message(t('Saved translate set: @added added - @deleted deleted', array('@added' => $stats['added'], '@deleted' => $stats['deleted'])));
}

/**
 * Submit callback: filter the manage translate set form.
 */
function translate_set_manage_filter_form_submit($form, &$form_state) {
  $form_state['redirect'] = array('admin/config/regional/translate/translate-set/manage-set', array(
    'query' => array(
      'textgroup' => $form_state['values']['textgroup'],
      'only_in_set' => $form_state['values']['only_in_set'],
    )
  ));
}

/**
 * Submit callback: reset the manage translate set form.
 */
function translate_set_manage_filter_form_reset($form, &$form_state) {
  $form_state['redirect'] = 'admin/config/regional/translate/translate-set/manage-set';
}

/**
 * Page callback: configure the settings.
 */
function translate_set_settings_form($form, $form_state) {

  $form['tracking_container'] = array(
    '#type' => 'fieldset',
    '#title' => t('Track strings'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );

  $form['tracking_container']['translate_set_tracking_enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable tracking of new strings'),
    '#default_value' => variable_get('translate_set_tracking_enabled', FALSE),
    '#description' => t('Tracking will be done when visiting the translate set screen.'),
  );

  $form['tracking_container']['translate_set_tracking_contexts'] = array(
    '#type' => 'textarea',
    '#title' => t('Contexts'),
    '#default_value' => variable_get('translate_set_tracking_contexts', ''),
    '#states' => array(
      'visible' => array(
        ':input[name="translate_set_tracking_enabled"]' => array('checked' => TRUE),
      ),
    ),
    '#description' => t('Enter contexts per line which will be used to evaluate whether a string will be added or not. Wildcards are possible. e.g.<br />custom<br />rules:rules_config:*:subject<br />taxonomy:*:name'),
  );

  $contexts = array();
  $records = db_select('locales_source', 'l')
    ->fields('l', array('context'))
    ->orderBy('context')
    ->condition('context', '', '!=')
    ->distinct()
    ->execute();
  foreach ($records as $record) {
    $contexts[] = '<li>' . check_plain($record->context) . '</li>';
  }

  if (!empty($contexts)) {
    $form['tracking_container']['translate_set_tracking_contexts_list'] = array(
      '#type' => 'fieldset',
      '#title' => t('Context list'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#states' => array(
        'visible' => array(
          ':input[name="translate_set_tracking_enabled"]' => array('checked' => TRUE),
        ),
      ),
    );
    $form['tracking_container']['translate_set_tracking_contexts_list']['list'] = array(
      '#type' => 'markup',
      '#markup' => '<ul>' . implode($contexts) . '</ul>',
    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );

  return $form;
}

/**
 * Validate callback: validates the translate set settings form.
 */
function translate_set_settings_form_validate($form, $form_state) {
  if ($form_state['values']['translate_set_tracking_enabled'] && empty($form_state['values']['translate_set_tracking_contexts'])) {
    form_set_error('translate_set_tracking_contexts', t('Tracking contexts is required.'));
  }
}

/**
 * Submit callback: saves the translate set settings form.
 */
function translate_set_settings_form_submit($form, $form_state) {

  if ($form_state['values']['translate_set_tracking_enabled']) {
    variable_set('translate_set_tracking_enabled', TRUE);
    variable_set('translate_set_tracking_contexts', $form_state['values']['translate_set_tracking_contexts']);
  }
  else {
    variable_del('translate_set_tracking_enabled');
    variable_del('translate_set_tracking_contexts');
  }

  drupal_set_message(t('The configuration options have been saved.'));
}
